@@ -42,6 +42,8 @@ struct KNSManager;
 rc_t KServiceMakeWithMgr(KService ** self, const struct VFSManager * vMgr,
     const struct KNSManager * mgr, const struct KConfig * kfg);
 
+rc_t KServiceResolve(KService * self, bool local, bool remote);
+
 rc_t KServiceNamesQueryExt ( KService * self, VRemoteProtocols protocols, 
     const char * cgi, const char * version, const char * outDir,
     const char * outFile, const KSrvResponse ** response );
@@ -64,6 +66,9 @@ rc_t KServiceTestNamesQueryExt ( KService * self, VRemoteProtocols protocols,
     const char * dir, const char * file, const char * expected );
 
 
+rc_t KServiceGetQuality(const KService * self, const char ** quality);
+
+
 rc_t KService1Search ( const struct KNSManager * mgr, const char * cgi,
     const char * acc, const struct Kart ** result );
 
