@@ -98,8 +98,8 @@
 
 /* In a future release of Bison, this section will be replaced
    by #include "samextract-tokens.h".  */
-#ifndef YY_SAM_HOME_VARTANIANMH_DEVEL_NCBI_VDB_LIBS_ALIGN_SAMEXTRACT_TOKENS_H_INCLUDED
-# define YY_SAM_HOME_VARTANIANMH_DEVEL_NCBI_VDB_LIBS_ALIGN_SAMEXTRACT_TOKENS_H_INCLUDED
+#ifndef YY_SAM_2T_DATA_ALTDEVEL_NCBI_VDB_LIBS_ALIGN_SAMEXTRACT_TOKENS_H_INCLUDED
+# define YY_SAM_2T_DATA_ALTDEVEL_NCBI_VDB_LIBS_ALIGN_SAMEXTRACT_TOKENS_H_INCLUDED
 /* Debug traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 0
@@ -203,7 +203,7 @@ extern YYSTYPE SAMlval;
 
 int SAMparse (SAMExtractor * state);
 
-#endif /* !YY_SAM_HOME_VARTANIANMH_DEVEL_NCBI_VDB_LIBS_ALIGN_SAMEXTRACT_TOKENS_H_INCLUDED  */
+#endif /* !YY_SAM_2T_DATA_ALTDEVEL_NCBI_VDB_LIBS_ALIGN_SAMEXTRACT_TOKENS_H_INCLUDED  */
 
 /* Copy the second part of user declarations.  */
 
