@@ -29,6 +29,8 @@
 
 #include <klib/rc.h>
 #include <klib/status.h>
+#include <klib/text.h> /* StringWhack */
+
 #include <cloud/aws.h>
 #include <cloud/gcp.h>
 
@@ -108,6 +110,38 @@ LIB_EXPORT const char * CC CloudProviderAsString(CloudProviderId cloud_provider)
 static
 CloudProviderId CloudMgrDetermineCurrentCloud ( const CloudMgr * self )
 {
+    rc_t rc = 0;
+    String * forced = NULL;
+
+    assert ( self );
+
+    rc = KConfigReadString ( self -> kfg, "/libs/cloud/provider", & forced );
+    if ( rc == 0 ) {
+        const char * a = NULL;
+        bool set = true;
+        CloudProviderId provider = cloud_provider_none;
+        assert ( forced && forced ->addr && forced -> addr [ 0 ] );
+        a = forced -> addr;
+        switch ( a [ 0 ] ) {
+        case 'a':
+            switch ( a [ 1 ] ) {
+            case 'w': provider = cloud_provider_aws  ; break;
+            case 'z': provider = cloud_provider_azure; break;
+            default : set = false;                     break;
+            }
+            break;
+        case 'g': provider = cloud_provider_gcp ; break;
+        case 'n': provider = cloud_provider_none; break;
+        default : set = false; break;
+        }
+
+        StringWhack ( forced );
+        forced = NULL;
+
+        if ( set )
+            return provider;
+    }
+
 #ifdef OUTSIDE_OF_CLOUD
     return cloud_provider_none;
 #endif
@@ -209,16 +243,17 @@ rc_t CloudMgrInit ( CloudMgr ** mgrp, const KConfig * kfg,
                     provider = CloudMgrDetermineCurrentCloud ( our_mgr );
                 }
 
-        if ( provider != cloud_provider_none )
+                if ( provider != cloud_provider_none )
                 {
-            TRACE ( "making current environment" );
-            rc = CloudMgrMakeCloud ( our_mgr, & our_mgr -> cur, provider );
-            if ( rc == 0 )
+                    TRACE ( "making current environment" );
+                    rc = CloudMgrMakeCloud (
+                        our_mgr, & our_mgr -> cur, provider );
+                    if ( rc == 0 )
                     {
-                TRACE ( "storing current environment" );
-                our_mgr -> cur_id = provider;
-            }
-        }
+                        TRACE ( "storing current environment" );
+                        our_mgr -> cur_id = provider;
+                    }
+                }
 
                 if ( rc == 0 )
                 {
@@ -233,6 +268,18 @@ rc_t CloudMgrInit ( CloudMgr ** mgrp, const KConfig * kfg,
     return rc;
 }
 
+#ifdef USE_SINGLETON		
+static bool SINGLETON = true;
+#else
+static bool SINGLETON = false;
+#endif
+
+bool CloudMgrUseSingleton(bool use) {
+    bool b = SINGLETON;
+    SINGLETON = use;
+    return b;
+}
+
 /* Make
  *  this is a singleton
  */
@@ -262,10 +309,10 @@ LIB_EXPORT rc_t CC CloudMgrMake ( CloudMgr ** mgrp,
 
                 /* try to set single-shot ( set once, never reset ) */
                 TRACE ( "attempting to set CloudMgr singleton" );
-		
-#ifdef USE_SINGLETON		
-                new_mgr = atomic_test_and_set_ptr ( & cloud_singleton, our_mgr, NULL );
-#endif
+
+                if (SINGLETON)
+                    new_mgr = atomic_test_and_set_ptr (
+                        & cloud_singleton, our_mgr, NULL );
 
                 /* if "new_mgr" is NULL, then our thread won the race */
                 if ( new_mgr == NULL )
