@@ -65,7 +65,7 @@ struct GCP;
 #include <sys/types.h>
 
 #include "cloud-cmn.h" /* KNSManager_Read */
-#include "cloud-priv.h"
+#include "cloud-priv.h" /* CloudGetCachedComputeEnvironmentToken */
 #include "gcp-priv.h" /* GCPAddAuthentication */
 
 #ifndef PATH_MAX
@@ -98,8 +98,8 @@ static char const *envCE()
     char const *const env = firstTime ? getenv(ENV_MAGIC_CE_TOKEN) : NULL;
     firstTime = false;
     if (env != NULL)
-        DBGMSG(DBG_VFS, DBG_FLAG(DBG_VFS_PATH), (
-            "Got location from environment\n"));
+        DBGMSG(DBG_VFS, DBG_FLAG(DBG_VFS_CE), (
+            "Got GCP location from environment\n"));
     return env;
 }
 
@@ -112,8 +112,8 @@ static rc_t readCE(GCP const *const self, size_t size, char location[])
         "http://metadata/computeMetadata/v1/instance/service-accounts/"
         "default/identity?audience=https://www.ncbi.nlm.nih.gov&format=full";
 
-    DBGMSG(DBG_VFS, DBG_FLAG(DBG_VFS_PATH),
-        ("Reading location from provider\n"));
+    DBGMSG(DBG_VFS, DBG_FLAG(DBG_VFS_CE),
+        ("Reading GCP location from provider\n"));
     return KNSManager_Read(self->dad.kns, location, size,
                          identityUrl, "Metadata-Flavor", "Google");
 }
@@ -132,11 +132,19 @@ rc_t CC GCPMakeComputeEnvironmentToken ( const GCP * self, const String ** ce_to
     else {
         char const *const env = envCE();
         char location[4096] = "";
-        rc_t const rc = env == NULL ? readCE(self, sizeof(location), location) : 0;
+        rc_t rc = 0;
+        if (CloudGetCachedComputeEnvironmentToken(&self->dad, ce_token))
+            return 0;
+        rc = env == NULL ? readCE(self, sizeof(location), location) : 0;
         if (rc == 0) {
             String s;
             StringInitCString(&s, env != NULL ? env : location);
-            return StringCopy(ce_token, &s);
+            rc = StringCopy(ce_token, &s);
+            if (rc == 0) { /* ignore rc below */
+                assert(ce_token);
+                CloudSetCachedComputeEnvironmentToken(&self->dad, *ce_token);
+            }
+            return rc;
         }
         return rc;
     }
@@ -779,8 +787,8 @@ LIB_EXPORT rc_t CC CloudMgrMakeGCP(const CloudMgr * self, GCP ** p_gcp)
                 &user_agrees_to_reveal_instance_identity);
         }
 
-        rc = CloudInit ( & gcp -> dad, ( const Cloud_vt * ) & GCP_vt_v1, "GCP", self -> kns, user_agrees_to_pay,
-            user_agrees_to_reveal_instance_identity );
+        rc = CloudInit ( & gcp -> dad, ( const Cloud_vt * ) & GCP_vt_v1, "GCP",
+            self, user_agrees_to_pay, user_agrees_to_reveal_instance_identity );
         if ( rc == 0 )
         {
             rc = PopulateCredentials(gcp);
