@@ -368,7 +368,7 @@ bool CC KMDataNodeInflate_v1 ( PBSTNode *n, void *data )
         pb -> rc = RC ( rcDB, rcMetadata, rcConstructing, rcMemory, rcExhausted );
         return true;
     }
-     
+
     b -> par = pb -> par;
     b -> meta = pb -> meta;
     b -> value = ( void* ) ( name + size + 1 );
@@ -378,7 +378,7 @@ bool CC KMDataNodeInflate_v1 ( PBSTNode *n, void *data )
     KRefcountInit ( & b -> refcount, 0, "KMDataNode", "inflate", name );
     b -> read_only = 0;
     strcpy ( b -> name, name );
-     
+
     /* a name with no associated value */
     if ( b -> vsize == 0 )
     {
@@ -413,7 +413,7 @@ rc_t KMDataNodeInflateAttr ( KMDataNode *n, bool byteswap )
     {
         KMDataNodeInflateData pb;
         size_t bst_size = PBSTreeSize ( bst );
-        
+
         pb . meta = n -> meta;
         pb . par = n;
         pb . bst = & n -> attr;
@@ -423,9 +423,9 @@ rc_t KMDataNodeInflateAttr ( KMDataNode *n, bool byteswap )
         pb . byteswap = byteswap;
         PBSTreeDoUntil ( bst, 0, KMAttrNodeInflate, & pb );
         rc = pb . rc;
-        
+
         PBSTreeWhack ( bst );
-        
+
         n -> value = ( char* ) n -> value + bst_size;
         n -> vsize -= bst_size;
     }
@@ -485,7 +485,7 @@ rc_t KMDataNodeInflateChild ( KMDataNode *n,
             KMDataNodeInflateData pb;
 
             pb . meta = n -> meta;
-            pb . par = n;        
+            pb . par = n;
             pb . bst = & n -> child;
             pb . node_size_limit = node_size_limit;
             pb . node_child_limit = node_child_limit;
@@ -494,9 +494,9 @@ rc_t KMDataNodeInflateChild ( KMDataNode *n,
             PBSTreeDoUntil ( bst, 0, KMDataNodeInflate, & pb );
             rc = pb . rc;
         }
-        
+
         PBSTreeWhack ( bst );
-        
+
         n -> value = ( char* ) n -> value + bst_size;
         n -> vsize -= bst_size;
     }
@@ -812,14 +812,15 @@ LIB_EXPORT rc_t CC KMDataNodeVOpenNodeRead ( const KMDataNode *self,
         int len;
 
         /* generate full path */
-        if ( args == NULL )
+        /* VDB-4386: cannot treat va_list as a pointer! */
+        /*if ( args == NULL )
             len = snprintf ( full, sizeof full, "%s", path );
-        else
+        else*/
             len = vsnprintf ( full, sizeof full, path, args );
         if ( len < 0 || len >= sizeof full )
             return RC ( rcDB, rcNode, rcOpening, rcPath, rcExcessive );
     }
-    
+
     rc = KMDataNodeFind ( self, & found, & p );
     if ( rc == 0 )
     {
@@ -932,9 +933,10 @@ LIB_EXPORT rc_t CC KMDataNodeVOpenNodeUpdate ( KMDataNode *self,
         int len;
 
         /* generate full path */
-        if ( args == NULL )
+        /* VDB-4386: cannot treat va_list as a pointer! */
+        /*if ( args == NULL )
             len = snprintf ( full, sizeof full, "%s", path );
-        else
+        else*/
             len = vsnprintf ( full, sizeof full, path, args );
         if ( len < 0 || len >= sizeof full )
             return RC ( rcDB, rcNode, rcOpening, rcPath, rcExcessive );
@@ -1136,14 +1138,14 @@ LIB_EXPORT rc_t CC KMDataNodeWrite ( KMDataNode *self, const void *buffer, size_
                 return RC ( rcDB, rcNode, rcWriting, rcMemory, rcExhausted );
             memmove ( value, buffer, size );
         }
-            
+
         if ( self -> value != NULL )
             free ( self -> value );
 
         self -> value = value;
         self -> vsize = size;
     }
-    
+
     self -> meta -> dirty = true;
 
     return 0;
@@ -1986,7 +1988,7 @@ LIB_EXPORT rc_t CC KMDataNodeDropChild ( KMDataNode *self, const char *path, ...
 
 LIB_EXPORT rc_t CC KMDataNodeVDropChild ( KMDataNode *self, const char *path, va_list args )
 {
-    int len;
+    int len = 0;
     rc_t rc;
     KMDataNode *found;
     char full [ 4096 ], *p = full;
@@ -2001,9 +2003,11 @@ LIB_EXPORT rc_t CC KMDataNodeVDropChild ( KMDataNode *self, const char *path, va
         return RC ( rcDB, rcNode, rcUpdating, rcPath, rcInvalid );
 
     /* generate full path */
-    if ( args == NULL )
+    /* VDB-4386: cannot treat va_list as a pointer! */
+    /*if ( args == NULL )
         len = snprintf ( full, sizeof full, "%s", path );
-    else
+    else*/
+    if ( path != NULL )
         len = vsnprintf ( full, sizeof full, path, args );
     if ( len < 0 || len >= sizeof full )
         return RC ( rcDB, rcNode, rcUpdating, rcPath, rcExcessive );
@@ -2650,7 +2654,7 @@ rc_t KMetadataMake ( KMetadata **metap,
                 * metap = meta;
                 return 0;
             }
-            
+
             rc = KMetadataPopulate ( meta, dir, path, read_only );
             if ( rc == 0 )
             {
@@ -2693,9 +2697,9 @@ rc_t KDBManagerOpenMetadataReadInt ( KDBManager *self,
 {
     char metapath [ 4096 ];
     rc_t rc = ( prerelease == 1 ) ?
-        KDirectoryVResolvePath ( wd, true, metapath, sizeof metapath, "meta", NULL ):
+        KDirectoryResolvePath_v1 ( wd, true, metapath, sizeof metapath, "meta" ):
         ( ( rev == 0 ) ?
-          KDirectoryVResolvePath ( wd, true, metapath, sizeof metapath, "md/cur", NULL ):
+          KDirectoryResolvePath_v1 ( wd, true, metapath, sizeof metapath, "md/cur" ):
           KDirectoryResolvePath ( wd, true, metapath, sizeof metapath, "md/r%.3u", rev ) );
     if(cached != NULL ) *cached = false;
     if ( rc == 0 )
@@ -2766,7 +2770,7 @@ rc_t KDBManagerOpenMetadataReadInt ( KDBManager *self,
         if ( rc == 0 )
         {
             rc = KMetadataMake ( & meta, ( KDirectory* ) wd, metapath, rev, true, true );
-            
+
             if ( rc == 0 )
             {
                 rc = KDBManagerInsertMetadata (self, meta );
@@ -2782,7 +2786,7 @@ rc_t KDBManagerOpenMetadataReadInt ( KDBManager *self,
 /*             rc = RC ( rcDB, rcMgr, rcOpening, rcMetadata, rcExists ); */
         }
     }
-    
+
     return rc;
 }
 
@@ -2944,7 +2948,7 @@ rc_t KDBManagerOpenMetadataUpdateInt ( KDBManager *self,
             KMetadataRelease ( meta );
         }
     }
-    
+
     return rc;
 }
 
@@ -3120,11 +3124,11 @@ LIB_EXPORT rc_t CC KMetadataMaxRevision ( const KMetadata *self, uint32_t *revis
             for ( rev_max = idx = 0; idx < count; ++ idx )
             {
                 const char *name;
-                    
+
                 rc = KNamelistGet ( listing, idx, & name );
                 if ( rc != 0 )
                     break;
-                    
+
                 if ( name [ 0 ] == 'r' )
                 {
                     char *end;
@@ -3136,7 +3140,7 @@ LIB_EXPORT rc_t CC KMetadataMaxRevision ( const KMetadata *self, uint32_t *revis
 
             * revision = rev_max;
         }
-                
+
         KNamelistRelease ( listing );
     }
     else if ( GetRCState ( rc ) == rcNotFound )
@@ -3195,7 +3199,7 @@ LIB_EXPORT rc_t CC KMetadataFreeze ( KMetadata *self )
     if ( self -> read_only )
         return RC ( rcDB, rcMetadata, rcFreezing, rcMetadata, rcReadonly );
 
-    switch ( KDirectoryVPathType ( self -> dir, "md/cur", NULL ) )
+    switch ( KDirectoryPathType_v1 ( self -> dir, "md/cur" ) )
     {
     case kptFile:
         break;
@@ -3237,7 +3241,7 @@ LIB_EXPORT rc_t CC KMetadataFreeze ( KMetadata *self )
                 KDirectoryRename ( self -> dir, false, revision, "md/cur" );
         }
     }
-        
+
     return rc;
 }
 
@@ -3314,7 +3318,7 @@ LIB_EXPORT rc_t CC KMetadataGetSequence ( const KMetadata *self,
         rc = KMDataNodeReadB64 ( found, val );
         KMDataNodeRelease ( found );
     }
-    
+
     return rc;
 }
 
@@ -3337,7 +3341,7 @@ LIB_EXPORT rc_t CC KMetadataSetSequence ( KMetadata *self,
         rc = KMDataNodeWriteB64 ( found, & val );
         KMDataNodeRelease ( found );
     }
-        
+
     return rc;
 }
 
@@ -3451,7 +3455,7 @@ rc_t KMDataNodeNamelistMake ( KNamelist **names, unsigned int count )
             * names = & self -> dad;
             return 0;
         }
-        
+
         free ( self );
     }
 
