@@ -280,9 +280,9 @@ rc_t KDBManagerMakeTableUpdate ( KDBManager *self,
                sealed
                skey
             */
-            rc = KDirectoryVSetAccess ( dir, true, 0220, 0222, ".", NULL );
+            rc = KDirectorySetAccess_v1 ( dir, true, 0220, 0222, "." );
             if ( rc == 0 )
-                rc = KDirectoryVCreateDir ( dir, 0775, kcmCreate, "md", NULL );
+                rc = KDirectoryCreateDir_v1 ( dir, 0775, kcmCreate, "md" );
             if ( rc == 0 )
             {
                 rc = KDirectoryRename ( dir, false, "meta", "md/r001" );
@@ -312,7 +312,7 @@ rc_t KDBManagerMakeTableUpdate ( KDBManager *self,
                 }
             }
             if ( rc == 0 )
-                rc = KDirectoryVCreateDir ( dir, 0775, kcmCreate, "idx", NULL );
+                rc = KDirectoryCreateDir_v1 ( dir, 0775, kcmCreate, "idx" );
             if ( rc == 0 )
             {
                 rc = KDirectoryRename ( dir, false, "skey", "idx/skey" );
@@ -328,12 +328,12 @@ rc_t KDBManagerMakeTableUpdate ( KDBManager *self,
             }
             if ( rc == 0 )
             {
-                if ( KDirectoryVPathType ( dir, "sealed", NULL ) == kptFile )
+                if ( KDirectoryPathType_v1 ( dir, "sealed" ) == kptFile )
                 {
                     rc = KDBLockDir ( dir, "." );
 #if ! V0_BACKWARD_COMPAT
                     if ( rc == 0 )
-                        KDirectoryVRemove ( dir, true, "sealed", NULL );
+                        KDirectoryRemove_v1 ( dir, true, "sealed" );
 #endif
                 }
             }
@@ -496,6 +496,20 @@ rc_t KDBManagerVCreateTableInt ( KDBManager *self,
     return rc;
 }
 
+static
+rc_t KDBManagerVCreateTableInt_noargs ( KDBManager *self,
+    KTable **tbl, KDirectory *wd, KCreateMode cmode, const char *path, ... )
+{
+    rc_t rc;
+    va_list args;
+
+    va_start ( args, path );
+    rc = KDBManagerVCreateTableInt ( self, tbl, wd, cmode, path, args );
+    va_end ( args );
+
+    return rc;
+}
+
 /* Default function is added only to make tools/kqsh build possible since
    it requires 1 to 1 mapping between K- and V-functions
 */
@@ -603,8 +617,8 @@ LIB_EXPORT rc_t CC KDatabaseVCreateTableByMask ( KDatabase *self,
             if ( (cmode_mask & kcmValueMask) != 0 )
                 cmode_mask |= kcmValueMask;
             table_cmode = (self->cmode & ~cmode_mask) | (cmode & cmode_mask);
-            rc = KDBManagerVCreateTableInt ( self -> mgr, tblp,
-                                             self -> dir, table_cmode, path, NULL );
+            rc = KDBManagerVCreateTableInt_noargs ( self -> mgr, tblp,
+                                             self -> dir, table_cmode, path );
             if ( rc == 0 )
             {
                 KTable *tbl = ( KTable* ) * tblp;
@@ -735,6 +749,21 @@ rc_t KDBManagerVOpenTableReadInt ( const KDBManager *cself,
     return rc;
 }
 
+static
+rc_t KDBManagerVOpenTableReadInt_noargs ( const KDBManager *cself,
+    const KTable **tblp, const KDirectory *wd, bool try_srapath,
+    const char *path, const struct VPath *vpath, ... )
+{
+    rc_t rc;
+    va_list args;
+
+    va_start ( args, vpath );
+    rc = KDBManagerVOpenTableReadInt ( cself, tblp, wd, try_srapath, path, args, vpath );
+    va_end ( args );
+
+    return rc;
+}
+
 LIB_EXPORT rc_t CC KDBManagerOpenTableRead ( const KDBManager *self,
     const KTable **tbl, const char *path, ... )
 {
@@ -773,7 +802,7 @@ LIB_EXPORT rc_t CC KDBManagerOpenTableReadVPath(const KDBManager *self,
     if (self == NULL)
         return RC(rcDB, rcMgr, rcOpening, rcSelf, rcNull);
 
-    return KDBManagerVOpenTableReadInt(self, tbl, self->wd, true, NULL, NULL, path);
+    return KDBManagerVOpenTableReadInt_noargs(self, tbl, self->wd, true, NULL, path);
 }
 
 LIB_EXPORT rc_t CC KDatabaseOpenTableRead ( const KDatabase *self,
@@ -810,8 +839,8 @@ LIB_EXPORT rc_t CC KDatabaseVOpenTableRead ( const KDatabase *self,
         path, sizeof path, "tbl", 3, name, args );
     if ( rc == 0 )
     {
-        rc = KDBManagerVOpenTableReadInt ( self -> mgr, tblp,
-                                           self -> dir, false, path, NULL, NULL );
+        rc = KDBManagerVOpenTableReadInt_noargs ( self -> mgr, tblp,
+                                           self -> dir, false, path, NULL );
         if ( rc == 0 )
         {
             KTable *tbl = ( KTable* ) * tblp;
@@ -945,6 +974,20 @@ rc_t KDBManagerVOpenTableUpdateInt ( KDBManager *self,
     return rc;
 }
 
+static
+rc_t KDBManagerVOpenTableUpdateInt_noargs ( KDBManager *self,
+    KTable **tbl, KDirectory *wd, const char *path, ... )
+{
+    rc_t rc;
+    va_list args;
+
+    va_start ( args, path );
+    rc = KDBManagerVOpenTableUpdateInt ( self, tbl, wd, path, args );
+    va_end ( args );
+
+    return rc;
+}
+
 LIB_EXPORT rc_t CC KDBManagerOpenTableUpdate ( KDBManager *self,
     KTable **tbl, const char *path, ... )
 {
@@ -1006,8 +1049,8 @@ LIB_EXPORT rc_t CC KDatabaseVOpenTableUpdate ( KDatabase *self,
         path, sizeof path, "tbl", 3, name, args );
     if ( rc == 0 )
     {
-        rc = KDBManagerVOpenTableUpdateInt ( self -> mgr, tblp,
-                                             self -> dir, path, NULL );
+        rc = KDBManagerVOpenTableUpdateInt_noargs ( self -> mgr, tblp,
+                                             self -> dir, path );
         if ( rc == 0 )
         {
             KTable *tbl = ( KTable* ) * tblp;
