@@ -199,7 +199,7 @@ rc_t KHttpFileMakeRequest ( const KHttpFile *cself, uint64_t pos, size_t req_siz
                 KClientHttpRequestSetCloudParams(req, cself -> need_env_token, cself -> payRequired);
                 if ( cself -> need_env_token && ! cself -> url_is_temporary)
                 {
-                    KClientHttpRequestAttachEnvironmentToken ( req );
+                    KClientHttpRequestAttachEnvironmentToken ( req, NULL );
                     /* TBD - there should be a version of POST that takes a timeout */
                     rc = KClientHttpRequestPOST ( req, rslt );
                 }
@@ -1348,7 +1348,8 @@ static rc_t KNSManagerVMakeHttpFileIntUnstableImpl( const KNSManager *self,
                         KClientHttpRequestSetCloudParams ( req, need_env_token, payRequired );
                         if ( need_env_token )
                         {
-                            KClientHttpRequestAttachEnvironmentToken ( req );
+                            KClientHttpRequestAttachEnvironmentToken ( req,
+                                NULL );
                         }
 
                         rc = KClientHttpRequestHEAD ( req, & rslt );
@@ -1512,6 +1513,19 @@ static rc_t KNSManagerVMakeHttpFileIntUnstableImpl( const KNSManager *self,
     return rc;
 }
 
+
+static rc_t KNSManagerVMakeHttpFileIntUnstableImpl_noargs( const KNSManager *self,
+    const KFile **file, KStream *conn, ver_t vers, bool reliable, bool need_env_token, bool payRequired, const KDataBuffer *buf,
+    const char *url, ... )
+{
+    va_list vl;
+    va_start( vl, url );
+    rc_t ret = KNSManagerVMakeHttpFileIntUnstableImpl(self, file, conn, vers,
+        reliable, need_env_token, payRequired, buf, url, vl);
+    va_end(vl);
+    return ret;
+}
+
 /******************************************************************************/
 
 rc_t KNSManagerVMakeHttpFileIntUnstableFromBuffer(const KNSManager *self,
@@ -1519,8 +1533,8 @@ rc_t KNSManagerVMakeHttpFileIntUnstableFromBuffer(const KNSManager *self,
     bool need_env_token, bool payRequired, const char *url,
     const KDataBuffer *buf)
 {
-    return KNSManagerVMakeHttpFileIntUnstableImpl(self, file, conn, vers,
-        reliable, need_env_token, payRequired, buf, url, NULL);
+    return KNSManagerVMakeHttpFileIntUnstableImpl_noargs(self, file, conn, vers,
+        reliable, need_env_token, payRequired, buf, url);
 }
 
 rc_t KNSManagerVMakeHttpFileIntUnstable(const KNSManager *self,
