@@ -84,6 +84,7 @@
 extern "C" {
 #endif
 
+struct Cloud;
 struct KFile;
 struct KNSManager;
 struct KClientHttp;
@@ -184,6 +185,9 @@ struct KClientHttp
     bool close_connection;
 
     EUriForm uf; /* Form of Request-URI in Request-Line when using proxy */
+
+    char * ua; /* user agent */
+    char * ua_head; /* user agent for HEAD */
 };
 
 void KClientHttpClose ( struct KClientHttp * self );
@@ -227,6 +231,9 @@ struct KClientHttpRequest
     bool payRequired; /* payment info required to access this URL */
 
     bool rangeRequested;
+    bool ceAdded;
+
+    bool head; /* is HEAD request */
 };
 
 void KClientHttpGetRemoteEndpoint ( const struct KClientHttp * self,
@@ -234,7 +241,8 @@ void KClientHttpGetRemoteEndpoint ( const struct KClientHttp * self,
 void KClientHttpGetLocalEndpoint ( const struct KClientHttp * self,
                                    struct KEndPoint * ep );
 
-rc_t KClientHttpRequestAttachEnvironmentToken( struct KClientHttpRequest * self );
+rc_t KClientHttpRequestAttachEnvironmentToken(
+    struct KClientHttpRequest * self, struct Cloud * cloud );
 
 /* exported private functions
 */
