@@ -795,6 +795,7 @@ static rc_t CC thread_func ( const KThread *self, void *data )
 }
 
 const int num_threads = 32;
+const size_t big_thread_size = ((size_t)(16u * 1024u * 1024u));
 
 FIXTURE_TEST_CASE ( CacheTee3_Multiple_Users_Multiple_Inst_1, CT3Fixture )
 {
@@ -812,7 +813,7 @@ FIXTURE_TEST_CASE ( CacheTee3_Multiple_Users_Multiple_Inst_1, CT3Fixture )
         td[ i ].cache_file = CACHEFILE;
         td[ i ].ram_pages = 0;
         td[ i ].fixture = this;
-        REQUIRE_RC ( KThreadMake ( &( t[ i ] ), thread_func, &( td[ i ] ) ) );
+        REQUIRE_RC ( KThreadMakeStackSize ( &( t[ i ] ), thread_func, &( td[ i ] ), big_thread_size ) );
     }
 
     for ( int i = 0; i < num_threads; ++i )
@@ -840,7 +841,7 @@ FIXTURE_TEST_CASE ( CacheTee3_Multiple_Users_Multiple_Inst_2, CT3Fixture )
         td[ i ].cache_file = NULL;
         td[ i ].ram_pages = 1000;
         td[ i ].fixture = this;
-        REQUIRE_RC ( KThreadMake ( &( t[ i ] ), thread_func, &( td[ i ] ) ) );
+        REQUIRE_RC ( KThreadMakeStackSize ( &( t[ i ] ), thread_func, &( td[ i ] ), big_thread_size ) );
     }
 
     for ( int i = 0; i < num_threads; ++i )
@@ -868,7 +869,7 @@ FIXTURE_TEST_CASE ( CacheTee3_Multiple_Users_Multiple_Inst_3, CT3Fixture )
         td[ i ].cache_file = CACHEFILE;
         td[ i ].ram_pages = 1000;
         td[ i ].fixture = this;
-        REQUIRE_RC ( KThreadMake ( &( t[ i ] ), thread_func, &( td[ i ] ) ) );
+        REQUIRE_RC ( KThreadMakeStackSize ( &( t[ i ] ), thread_func, &( td[ i ] ), big_thread_size ) );
     }
 
     for ( int i = 0; i < num_threads; ++i )
@@ -1272,6 +1273,60 @@ FIXTURE_TEST_CASE ( CacheTee3_request_record, CT3Fixture )
 }
 #endif
 
+typedef struct ThreadData2
+{
+    VFSManager * mgr;
+    const char * url;
+    uint64_t exp_size;
+    uint32_t exp_magic;
+    size_t num_errors;
+} ThreadData2;
+
+static rc_t CC thread_func2 ( const KThread *self, void *data )
+{
+    ThreadData2 * td = ( ThreadData2 * )data;
+
+    VPath* path = 0;
+    const KFile* file = 0;
+    uint64_t size = 0;
+    char buffer[ 20 * 1024 ];
+
+    if ( 0 != VFSManagerMakePath( td -> mgr, &path, td -> url ) )
+       ( td -> num_errors )++;
+    else
+    {
+        if ( 0 != VFSManagerOpenFileRead( td -> mgr, &file, path ) )
+            ( td -> num_errors)++;
+        else
+        {
+            if ( 0 != KFileSize( file, &size ) )
+               ( td -> num_errors)++;
+            else
+            {
+                if ( 0 != KFileReadExactly( file, 0, &buffer, sizeof( buffer ) ) )
+                    ( td -> num_errors)++;
+                else
+                {
+                    uint32_t magic = 0;
+                    memcpy( &magic, buffer, sizeof( magic ) );
+
+                    if ( size != td -> exp_size )
+                        ( td -> num_errors )++;
+                    if ( magic != td -> exp_magic )
+                        ( td -> num_errors )++;
+                }
+            }
+
+            if ( 0 != KFileRelease( file ) )
+                ( td -> num_errors )++;
+        }
+        if ( 0 != VPathRelease( path ) )
+            ( td -> num_errors )++;
+   }
+
+    return 0;
+}
+
 TEST_CASE( concurrent_reads_from_different_files )
 {
     KOutMsg ( "concurrent reads from different files\n" );
@@ -1304,12 +1359,41 @@ TEST_CASE( concurrent_reads_from_different_files )
         REQUIRE_RC( VPathRelease( path ) );
     }
 
-    const size_t NUM_PASSES = 2;
+    // const size_t NUM_PASSES = 2;
     const size_t NUM_THREADS = 8;
+    
+    ThreadData2 data[ NUM_THREADS ];
+    for ( size_t i = 0; i < NUM_THREADS; ++i )
+    {
+        size_t url_index = ( i % URL_COUNT );
+        data[ i ] . mgr = mgr;
+        data[ i ] . url = urls[ url_index ] . c_str();
+        data[ i ] . exp_size = sizes[ url_index ];
+        data[ i ] . exp_magic = magics[ url_index ];
+        data[ i ] . num_errors = 0;
+    }
+
+    KThread * threads[ NUM_THREADS ];
+    for ( size_t i = 0; i < NUM_THREADS; ++i )
+    {
+        REQUIRE_RC ( KThreadMakeStackSize ( &( threads[ i ] ), thread_func2, &( data[ i ] ), big_thread_size ) );
+    }
+
+    //std::this_thread::sleep_for( std::chrono::milliseconds( 1000 ) );
+
+    for ( size_t i = 0; i < NUM_THREADS; ++i )
+    {
+        rc_t rc_thread;
+        REQUIRE_RC ( KThreadWait ( threads[ i ], &rc_thread ) );
+        REQUIRE_RC ( rc_thread );
+        REQUIRE_RC ( KThreadRelease ( threads[ i ] ) );
+    }
+
+    /*
     size_t thread_errors[ NUM_THREADS ];
     for ( size_t i = 0; i < NUM_THREADS; ++i )
         thread_errors[ i ] = 0;
-    
+
     for ( size_t pass = 0; pass < NUM_PASSES; ++pass )
     {
         vector< thread > tt( NUM_THREADS );
@@ -1362,10 +1446,14 @@ TEST_CASE( concurrent_reads_from_different_files )
             tt[ i ].join();
         }
     }
-
     size_t all_errors = 0;
     for ( size_t i = 0; i < NUM_THREADS; ++i )
         all_errors += thread_errors[ i ];
+    */
+
+    size_t all_errors = 0;
+    for ( size_t i = 0; i < NUM_THREADS; ++i )
+        all_errors += data[ i ] . num_errors;
     REQUIRE( all_errors == 0  );
     
     REQUIRE_RC( VFSManagerRelease( mgr ) );
@@ -1408,7 +1496,10 @@ rc_t CC KMain ( int argc, char *argv[] )
         srand ( time ( NULL ) );
         KConfigDisableUserSettings ();
         rc = CacheTee3Tests ( argc, argv );
-        KOutMsg ( "and the result is: %R\n", rc );
+        if ( (rc_t)-1 != rc )
+            KOutMsg ( "and the result is: %R\n", rc );
+        else
+            KOutMsg ( "and the result is: %d\n", rc );
         ArgsWhack ( args );
     }
     return rc;
