@@ -66,6 +66,15 @@ TEST_CASE(Token_Construct)
     REQUIRE ( ! st . IsFake () );
 }
 
+TEST_CASE(Token_Construct_WithLocation)
+{
+    Token :: Location loc ("qq", 1, 2 );
+    Token t ( KW_virtual, "virtual", loc );
+    REQUIRE_EQ ( string ( loc . m_file ), string ( t . GetLocation () . m_file ) );
+    REQUIRE_EQ ( loc . m_line, t . GetLocation () . m_line );
+    REQUIRE_EQ ( loc . m_column, t . GetLocation () . m_column );
+}
+
 TEST_CASE(Token_Construct_Fake)
 {
     Token t ( 42 );
@@ -206,478 +215,532 @@ PrintParseTree ( const ParseTree * p_t, ostream& p_out )
 }
 
 static
-bool
-ParseAndVerify ( const char* p_source )
+void
+ParseAndVerify ( const char* p_source, bool p_debug = false )
 {
     SchemaParser p;
-    if ( ! p . ParseString ( p_source ) )
+    if ( ! p . ParseString ( p_source, p_debug ) )
     {
-        return false;
+        throw logic_error ( string ( "ParseAndVerify : ParseString() failed: " ) + p . GetErrors () . GetMessageText ( 0 ) );
     }
     ostringstream str;
     PrintParseTree ( p . GetParseTree (), str );
-    if ( str . str () == p_source )
+    if ( str . str () != p_source )
     {
-        return true;
+        cout << "expected: '" << p_source << "'" << endl
+             << "actual:   '" << str . str () << "'" << endl;
+        throw logic_error ( "ParseAndVerify failed: " );
     }
-    cout << "expected: '" << p_source << "'" << endl
-         << "actual:   '" << str . str () << "'" << endl;
-    return false;
 }
 
 TEST_CASE ( EmptyInput )
 {
-    REQUIRE ( ParseAndVerify ( "" ) );
+    ParseAndVerify ( "" );
 }
 
 TEST_CASE ( Version1 )
 {
-    REQUIRE ( ParseAndVerify ( "version 1; include \"qq\";" ) );
+    ParseAndVerify ( "version 1; include \"qq\";" );
 }
 
 TEST_CASE ( MultipleDecls )
 {
-    REQUIRE ( ParseAndVerify ( "version 1; include \"qq\"; include \"aa\";" ) );
+    ParseAndVerify ( "version 1; include \"qq\"; include \"aa\";" );
 }
 
 TEST_CASE ( Typedef )
 {
-    REQUIRE ( ParseAndVerify ( "typedef ns:oldName newName;" ) );
+    ParseAndVerify ( "typedef ns:oldName newName;" );
 }
 TEST_CASE ( Keywords_as_identifiers )
 {
-    REQUIRE ( ParseAndVerify (
-        "typedef ns:database:decode:encode:read:table:type:view:write newName;" ) );
+    ParseAndVerify (
+        "typedef ns:database:decode:encode:read:table:type:view:write newName;" );
 }
 
 TEST_CASE ( TypedefDim )
 {
-    REQUIRE ( ParseAndVerify ( "typedef oldName newName [ 12 ];" ) );
+    ParseAndVerify ( "typedef oldName newName [ 12 ];" );
 }
 TEST_CASE ( TypedefVarDim )
 {
-    REQUIRE ( ParseAndVerify ( "typedef oldName newName [ * ];" ) );
+    ParseAndVerify ( "typedef oldName newName [ * ];" );
 }
 
 TEST_CASE ( TypedefMultipleNames )
 {
-    REQUIRE ( ParseAndVerify ( "typedef oldName neawName1, newName2 [ 12 ], newName3;" ) );
+    ParseAndVerify ( "typedef oldName neawName1, newName2 [ 12 ], newName3;" );
 }
 
 TEST_CASE ( Typeset )
 {
-    REQUIRE ( ParseAndVerify ( "typeset newName { a, b[1], c };" ) );
+    ParseAndVerify ( "typeset newName { a, b[1], c };" );
 }
 
 TEST_CASE ( Format )
 {
-    REQUIRE ( ParseAndVerify ( "fmtdef newName;" ) );
+    ParseAndVerify ( "fmtdef newName;" );
 }
 TEST_CASE ( FormatRename )
 {
-    REQUIRE ( ParseAndVerify ( "fmtdef oldName newName;" ) );
+    ParseAndVerify ( "fmtdef oldName newName;" );
 }
 
 TEST_CASE ( Const )
 {
-    REQUIRE ( ParseAndVerify ( "const t c = 1;" ) );
+    ParseAndVerify ( "const t c = 1;" );
 }
 TEST_CASE ( ConstDim )
 {
-    REQUIRE ( ParseAndVerify ( "const t[2] c = 1;" ) );
+    ParseAndVerify ( "const t[2] c = 1;" );
 }
 
 TEST_CASE ( Alias )
 {
-    REQUIRE ( ParseAndVerify ( "alias a b;" ) );
+    ParseAndVerify ( "alias a b;" );
 }
 
 TEST_CASE ( Extern_AndUntyped )
 {
-    REQUIRE ( ParseAndVerify ( "extern function __untyped fn ();" ) );
+    ParseAndVerify ( "extern function __untyped fn ();" );
 }
 
 TEST_CASE ( Function_RowLength )
 {
-    REQUIRE ( ParseAndVerify ( "function __row_length fn ();" ) );
+    ParseAndVerify ( "function __row_length fn ();" );
 }
 
 TEST_CASE ( Function_Naked )
 {
-    REQUIRE ( ParseAndVerify ( "function t fn ( a b );" ) );
+    ParseAndVerify ( "function t fn ( a b );" );
 }
 
 TEST_CASE ( Function_ArrayReturn )
 {
-    REQUIRE ( ParseAndVerify ( "function t[3] fn ( a b );" ) );
+    ParseAndVerify ( "function t[3] fn ( a b );" );
 }
 
 TEST_CASE ( Function_Schema )
 {
-    REQUIRE ( ParseAndVerify ( "function < type b,  a / fmt c > t fn ( a b );" ) );
+    ParseAndVerify ( "function < type b,  a / fmt c > t fn ( a b );" );
 }
 
 TEST_CASE ( Function_Factory )
 {
-    REQUIRE ( ParseAndVerify ( "function t fn < a b, c d > ( a b );" ) );
+    ParseAndVerify ( "function t fn < a b, c d > ( a b );" );
 }
 
 TEST_CASE ( Function_NoFormals )
 {
-    REQUIRE ( ParseAndVerify ( "function t fn ();" ) );
+    ParseAndVerify ( "function t fn ();" );
 }
 
 TEST_CASE ( Function_Formals_OptionalOnly )
 {
-    REQUIRE ( ParseAndVerify ( "function t fn ( * a b );" ) );
+    ParseAndVerify ( "function t fn ( * a b );" );
 }
 TEST_CASE ( Function_Formals_MandatoryAndOptional_NoComma )
 {
-    REQUIRE ( ParseAndVerify ( "function t fn ( a b * a b );" ) );
+    ParseAndVerify ( "function t fn ( a b * a b );" );
 }
 TEST_CASE ( Function_Formals_MandatoryAndOptional )
 {
-    REQUIRE ( ParseAndVerify ( "function t fn ( a b, * a b );" ) );
+    ParseAndVerify ( "function t fn ( a b, * a b );" );
 }
 TEST_CASE ( Function_Formals_MandatoryAndVariadic )
 {
-    REQUIRE ( ParseAndVerify ( "function t fn ( a b, ...  );" ) );
+    ParseAndVerify ( "function t fn ( a b, ...  );" );
 }
 TEST_CASE ( Function_Formals_MandatoryOptionalAndVariadic )
 {
-    REQUIRE ( ParseAndVerify ( "function t fn ( a b, * a b, ...  );" ) );
+    ParseAndVerify ( "function t fn ( a b, * a b, ...  );" );
 }
 TEST_CASE ( Function_Formals_Control )
 {
-    REQUIRE ( ParseAndVerify ( "function t fn ( control a b );" ) );
+    ParseAndVerify ( "function t fn ( control a b );" );
 }
 
 TEST_CASE ( Function_Prologue_Rename )
 {
-    REQUIRE ( ParseAndVerify ( "function t fn ( a b ) = fn;" ) );
+    ParseAndVerify ( "function t fn ( a b ) = fn;" );
 }
 TEST_CASE ( Function_Prologue_Script_Return )
 {
-    REQUIRE ( ParseAndVerify ( "schema t fn ( a b ) { return 1; };" ) );
+    ParseAndVerify ( "schema t fn ( a b ) { return 1; };" );
 }
 TEST_CASE ( Function_Prologue_Script_Assign )
 {
-    REQUIRE ( ParseAndVerify ( "schema function t fn ( a b ) { a c = 1; };" ) );
+    ParseAndVerify ( "schema function t fn ( a b ) { a c = 1; };" );
 }
 
 TEST_CASE ( Script )
 {
-    REQUIRE ( ParseAndVerify ( "schema t fn ( a b );" ) );
+    ParseAndVerify ( "schema t fn ( a b );" );
 }
 TEST_CASE ( Script_Function )
 {
-    REQUIRE ( ParseAndVerify ( "schema function t fn ( a b );" ) );
+    ParseAndVerify ( "schema function t fn ( a b );" );
 }
 
 TEST_CASE ( Validate )
 {
-    REQUIRE ( ParseAndVerify ( "validate function t fn ( a b );" ) );
+    ParseAndVerify ( "validate function t fn ( a b );" );
 }
 
 TEST_CASE ( Physical_Shorthand )
 {
-    REQUIRE ( ParseAndVerify ( "physical t fn #1.0 = { return 1; };" ) );
+    ParseAndVerify ( "physical t fn #1.0 = { return 1; };" );
 }
 
 TEST_CASE ( Physical_Longhand )
 {
-    REQUIRE ( ParseAndVerify (
-        "physical t fn #1.0 { decode { return 1; } ; encode { return 1; } ; __row_length = f () };" ) );
+    ParseAndVerify (
+        "physical t fn #1.0 { decode { return 1; } ; encode { return 1; } ; __row_length = f () };" );
 }
 
 TEST_CASE ( Physical_noheader )
 {
-    REQUIRE ( ParseAndVerify (
-        "physical __no_header t fn #1.0 = { return a; };" ) );
+    ParseAndVerify (
+        "physical __no_header t fn #1.0 = { return a; };" );
 }
 
 TEST_CASE ( Table_NoParents )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1.1.1 { t a = 1; };" ) );
+    ParseAndVerify ( "table t #1.1.1 { t a = 1; };" );
 }
 TEST_CASE ( Table_Parents )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1.1.1 = t1, t2, t3 { t a = 1; };" ) );
+    ParseAndVerify ( "table t #1.1.1 = t1, t2, t3 { t a = 1; };" );
 }
 TEST_CASE ( Table_ParentsWithVersion )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1.1.1 = t1, t2, t3#1.2.3 { t a = 1; };" ) );
+    ParseAndVerify ( "table t #1.1.1 = t1, t2, t3#1.2.3 { t a = 1; };" );
 }
 
 TEST_CASE ( Table_Empty )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 {};" ) );
+    ParseAndVerify ( "table t #1 {};" );
 }
 TEST_CASE ( Table_ProdStmt )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { t a = 1; };" ) );
+    ParseAndVerify ( "table t #1 { t a = 1; };" );
 }
 TEST_CASE ( Table_ProdStmt_FunCallSchemaNoFormals )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { t p = < t, 1 > fn < 1 > (); };" ) );
+    ParseAndVerify ( "table t #1 { t p = < t, 1 > fn < 1 > (); };" );
 }
 
 TEST_CASE ( Table_Column )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { column t c; };" ) );
+    ParseAndVerify ( "table t #1 { column t c; };" );
 }
 TEST_CASE ( Table_Column_PhysicalEncoding)
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { column <1> p c; };" ) );
+    ParseAndVerify ( "table t #1 { column <1> p c; };" );
 }
 TEST_CASE ( Table_Column_PhysicalEncoding_WithVersion )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { column <1> p#1.2.3 c; };" ) );
+    ParseAndVerify ( "table t #1 { column <1> p#1.2.3 c; };" );
 }
 TEST_CASE ( Table_Column_PhysicalEncoding_WithFactory )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { column <1> p <1> c; };" ) );
+    ParseAndVerify ( "table t #1 { column <1> p <1> c; };" );
 }
 TEST_CASE ( Table_Column_PhysicalEncoding_WithVersionAndFactory )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { column <1> p#1 <1> c; };" ) );
+    ParseAndVerify ( "table t #1 { column <1> p#1 <1> c; };" );
 }
 TEST_CASE ( Table_Column_Default )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { default column t c; };" ) );
+    ParseAndVerify ( "table t #1 { default column t c; };" );
 }
 TEST_CASE ( Table_Column_Extern_WithPhysical )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { extern column < U32 > izip_encoding #1 CHANNEL; };" ) );
+    ParseAndVerify ( "table t #1 { extern column < U32 > izip_encoding #1 CHANNEL; };" );
 }
 TEST_CASE ( Table_Column_Extern_WithNakedPhysical )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { extern column bool_encoding #1 HIGH_QUALITY; };" ) );
+    ParseAndVerify ( "table t #1 { extern column bool_encoding #1 HIGH_QUALITY; };" );
 }
 TEST_CASE ( Table_Column_Extern_WithPhysicalFactory )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 {  column F32_4ch_encoding < 24 > BASE_FRACTION; };" ) );
+    ParseAndVerify ( "table t #1 {  column F32_4ch_encoding < 24 > BASE_FRACTION; };" );
 }
 
 TEST_CASE ( Table_Column_Readonly )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { readonly column t c; };" ) );
+    ParseAndVerify ( "table t #1 { readonly column t c; };" );
 }
 TEST_CASE ( Table_Column_AllMods )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { default extern readonly column t c; };" ) );
+    ParseAndVerify ( "table t #1 { default extern readonly column t c; };" );
 }
 
 TEST_CASE ( Table_Column_limit )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { column limit = 1; };" ) );
+    ParseAndVerify ( "table t #1 { column limit = 1; };" );
 }
 TEST_CASE ( Table_Column_default_limit )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { column default limit = 1; };" ) );
+    ParseAndVerify ( "table t #1 { column default limit = 1; };" );
 }
 
 TEST_CASE ( Table_Column_withBody_1 )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { column t c { read = 1 | 2; validate = 2 | 3; limit = 100; }; };" ) );
+    ParseAndVerify ( "table t #1 { column t c { read = 1 | 2; validate = 2 | 3; limit = 100; }; };" );
 }
 TEST_CASE ( Table_Column_withBody_2 )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { column t c { read = 1 | 2; validate = 2 | 3; limit = 100; } };" ) );
+    ParseAndVerify ( "table t #1 { column t c { read = 1 | 2; validate = 2 | 3; limit = 100; } };" );
 }
 TEST_CASE ( Table_Column_withExpr )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { column t c = 0 | 1; };" ) );
+    ParseAndVerify ( "table t #1 { column t c = 0 | 1; };" );
 }
 
 TEST_CASE ( Table_DefaultView )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { default view \"QQ\"; };" ) );
+    ParseAndVerify ( "table t #1 { default view \"QQ\"; };" );
 }
 
 TEST_CASE ( Table_PhysMbr_Static )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { static t .c; };" ) );
+    ParseAndVerify ( "table t #1 { static t .c; };" );
 }
 TEST_CASE ( Table_PhysMbr_StaticWithInit )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { static t .c = 1; };" ) );
+    ParseAndVerify ( "table t #1 { static t .c = 1; };" );
 }
 TEST_CASE ( Table_PhysMbr_Physical )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { physical t .c = 1; };" ) );
+    ParseAndVerify ( "table t #1 { physical t .c = 1; };" );
 }
 TEST_CASE ( Table_PhysMbr_PhysicalWithVersion )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { physical column NCBI #1 .CLIP_ADAPTER_LEFT; };" ) );
+    ParseAndVerify ( "table t #1 { physical column NCBI #1 .CLIP_ADAPTER_LEFT; };" );
 }
 
 TEST_CASE ( Table_PhysMbr_StaticPhysical )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { static physical t .c = 1; };" ) );
+    ParseAndVerify ( "table t #1 { static physical t .c = 1; };" );
 }
 TEST_CASE ( Table_PhysMbr_WithColumn )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { static column t .c = 1; };" ) );
+    ParseAndVerify ( "table t #1 { static column t .c = 1; };" );
 }
 
 TEST_CASE ( Table_PhysMbr_WithSchema )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { static column <1, 2> t .c = 1; };" ) );
+    ParseAndVerify ( "table t #1 { static column <1, 2> t .c = 1; };" );
 }
 TEST_CASE ( Table_PhysMbr_WithVersion )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { static column t#1 .c = 1; };" ) );
+    ParseAndVerify ( "table t #1 { static column t#1 .c = 1; };" );
 }
 TEST_CASE ( Table_PhysMbr_WithFactory )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { static column t<1> .c = 1; };" ) );
+    ParseAndVerify ( "table t #1 { static column t<1> .c = 1; };" );
 }
 TEST_CASE ( Table_PhysMbr_WithAll )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { static column <1, t = a:b> t #2.3.4 < 5 > .c = 1; };" ) );
+    ParseAndVerify ( "table t #1 { static column <1, t = a:b> t #2.3.4 < 5 > .c = 1; };" );
 }
 
 TEST_CASE ( Table_Untyped )
 {
-    REQUIRE ( ParseAndVerify ( "table t #1 { __untyped = a:b(); };" ) );
+    ParseAndVerify ( "table t #1 { __untyped = a:b(); };" );
 }
 
 TEST_CASE ( Database_Empty )
 {
-    REQUIRE ( ParseAndVerify ( "database a:b #1.2.3 {};" ) );
+    ParseAndVerify ( "database a:b #1.2.3 {};" );
 }
 TEST_CASE ( Database_WithParent )
 {
-    REQUIRE ( ParseAndVerify ( "database a:b #1.2.3 = ns:dad #4.5.6 {};" ) );
+    ParseAndVerify ( "database a:b #1.2.3 = ns:dad #4.5.6 {};" );
 }
 
 TEST_CASE ( Database_DbMember )
 {
-    REQUIRE ( ParseAndVerify ( "database d#1 { database ns : db DB; };" ) );
+    ParseAndVerify ( "database d#1 { database ns : db DB; };" );
 }
 TEST_CASE ( Database_DbMember_WithTemplate )
 {
-    REQUIRE ( ParseAndVerify ( "database d#1 { template database ns : db DB; };" ) );
+    ParseAndVerify ( "database d#1 { template database ns : db DB; };" );
 }
 
 TEST_CASE ( Database_TableMember )
 {
-    REQUIRE ( ParseAndVerify ( "database d#1 { table ns : tbl T; };" ) );
+    ParseAndVerify ( "database d#1 { table ns : tbl T; };" );
 }
 TEST_CASE ( Database_TableMember_WithTemplate )
 {
-    REQUIRE ( ParseAndVerify ( "database d#1 { template table ns : tbl T; };" ) );
+    ParseAndVerify ( "database d#1 { template table ns : tbl T; };" );
 }
 
 TEST_CASE ( Database_MultipleMembers )
 {
-    REQUIRE ( ParseAndVerify ( "database d#1 { database ns : db DB; table ns : tbl T; };" ) );
+    ParseAndVerify ( "database d#1 { database ns : db DB; table ns : tbl T; };" );
 }
 
 TEST_CASE ( Include )
 {
-    REQUIRE ( ParseAndVerify ( "include 'insdc/sra.vschema';" ) );
+    ParseAndVerify ( "include 'insdc/sra.vschema';" );
 }
 
 // Expressions
 TEST_CASE ( CastExpr )
 {
-    REQUIRE ( ParseAndVerify ( "function t f() { return (a)b; };" ) );
+    ParseAndVerify ( "function t f() { return (a)b; };" );
 }
 
 TEST_CASE ( AtExpr )
 {
-    REQUIRE ( ParseAndVerify ( "function t f() { return @; };" ) );
+    ParseAndVerify ( "function t f() { return @; };" );
 }
 
 TEST_CASE ( HexExpr )
 {
-    REQUIRE ( ParseAndVerify ( "function t f() { return 0x1290ABEF; };" ) );
+    ParseAndVerify ( "function t f() { return 0x1290ABEF; };" );
 }
 
 TEST_CASE ( FloatExpr )
 {
-    REQUIRE ( ParseAndVerify ( "function t f() { return 1.2; };" ) );
+    ParseAndVerify ( "function t f() { return 1.2; };" );
 }
 
 TEST_CASE ( ExpFloatExpr )
 {
-    REQUIRE ( ParseAndVerify ( "function t f() { return 1E2; };" ) );
+    ParseAndVerify ( "function t f() { return 1E2; };" );
 }
 
 TEST_CASE ( StringExpr )
 {
-    REQUIRE ( ParseAndVerify ( "function t f() { return \"qq\"; };" ) );
+    ParseAndVerify ( "function t f() { return \"qq\"; };" );
 }
 
 TEST_CASE ( EscapedStringExpr )
 {
-    REQUIRE ( ParseAndVerify ( "function t f() { return \"q\\q\"; };" ) );
+    ParseAndVerify ( "function t f() { return \"q\\q\"; };" );
 }
 
 TEST_CASE ( ConstVectExpr_Empty )
 {
-    REQUIRE ( ParseAndVerify ( "function t f() { return [ ]; };" ) );
+    ParseAndVerify ( "function t f() { return [ ]; };" );
 }
 TEST_CASE ( ConstVectExpr )
 {
-    REQUIRE ( ParseAndVerify ( "function t f() { return [ 1, 2, 3 ]; };" ) );
+    ParseAndVerify ( "function t f() { return [ 1, 2, 3 ]; };" );
 }
 
 TEST_CASE ( BoolExpr_True )
 {
-    REQUIRE ( ParseAndVerify ( "function t f() { return true; };" ) );
+    ParseAndVerify ( "function t f() { return true; };" );
 }
 TEST_CASE ( BoolExpr_False )
 {
-    REQUIRE ( ParseAndVerify ( "function t f() { return false; };" ) );
+    ParseAndVerify ( "function t f() { return false; };" );
 }
 
 TEST_CASE ( NegateExpr_Ident )
 {
-    REQUIRE ( ParseAndVerify ( "function t f() { return -a; };" ) );
+    ParseAndVerify ( "function t f() { return -a; };" );
 }
 TEST_CASE ( NegateExpr_Int )
 {
-    REQUIRE ( ParseAndVerify ( "function t f() { return -1; };" ) );
+    ParseAndVerify ( "function t f() { return -1; };" );
 }
 TEST_CASE ( NegateExpr_Float )
 {
-    REQUIRE ( ParseAndVerify ( "function t f() { return -1.0; };" ) );
+    ParseAndVerify ( "function t f() { return -1.0; };" );
 }
 
 TEST_CASE ( FuncExpr_NoSchemaNoFactNoParms )
 {
-    REQUIRE ( ParseAndVerify ( "function t f() { return f(); };" ) );
+    ParseAndVerify ( "function t f() { return f(); };" );
 }
 TEST_CASE ( FuncExpr_NoSchemaFactNoParms )
 {
-    REQUIRE ( ParseAndVerify ( "function t f() { return f<1>(); };" ) );
+    ParseAndVerify ( "function t f() { return f<1>(); };" );
 }
 TEST_CASE ( FuncExpr_NoSchemaNoFactParms )
 {
-    REQUIRE ( ParseAndVerify ( "function t f() { return f(1); };" ) );
+    ParseAndVerify ( "function t f() { return f(1); };" );
 }
 TEST_CASE ( FuncExpr_NoSchemaFactParms )
 {
-    REQUIRE ( ParseAndVerify ( "function t f() { return f<a, b>(c, d); };" ) );
+    ParseAndVerify ( "function t f() { return f<a, b>(c, d); };" );
 }
 TEST_CASE ( FuncExpr_Version )
 {
-    REQUIRE ( ParseAndVerify ( "function t f() { return f#1<a, b>(c, d); };" ) );
+    ParseAndVerify ( "function t f() { return f#1<a, b>(c, d); };" );
 }
 
 TEST_CASE ( PhysicalIdent )
 {
-    REQUIRE ( ParseAndVerify ( "function t f() { return .b; };" ) );
+    ParseAndVerify ( "function t f() { return .b; };" );
 }
 
 // Version 2
 
 TEST_CASE ( VersionOther )
 {
-    REQUIRE ( ParseAndVerify ( "version 3.14; $" ) ); //TODO
+    REQUIRE ( ! SchemaParser () . ParseString ( "version 3.14; $" ) );
+}
+
+TEST_CASE ( Version2_Decl1 )
+{
+    ParseAndVerify ( "version 2; include \"qq\";" );
+}
+
+TEST_CASE ( View_InVers1 )
+{
+    REQUIRE ( ! SchemaParser () . ParseString ( "view X#1 < T t > {};" ) );
+}
+
+TEST_CASE ( Decls_1_And_2 )
+{
+    ParseAndVerify ( "version 2;  include \"qq\"; view X#1< T t > { U8 p = 1; };" );
+}
+
+TEST_CASE ( View_OneParam_OneStmt )
+{
+    ParseAndVerify ( "version 2; view X#1 < T t > { U8 p = 1; };" );
+}
+
+TEST_CASE ( View_Params_Stmts )
+{
+    ParseAndVerify ( "version 2; view X#1 < T1 t1, T2 t2 > { U8 p = 1; ; column U32 c = p; };" );
+}
+
+TEST_CASE ( View_OneParent )
+{
+    ParseAndVerify ( "version 2; view X#1 < T t > = V < t > { U8 p = 1; };" );
+}
+
+TEST_CASE ( View_TwoParents )
+{
+    ParseAndVerify ( "version 2; view X#1 < T t > = V#1<t>,W<t> { U8 p = 1; };" );
+}
+
+TEST_CASE ( View_OneParam_EmptyBody )
+{
+    ParseAndVerify ( "version 2; view X#1 < T t > {};" );
+}
+
+TEST_CASE ( View_NoPhysicalColumns )
+{
+    REQUIRE ( ! SchemaParser () . ParseString ( "version 2; view X#1 < T t > { column U8 .PHYS = 1; };" ) );
+}
+
+TEST_CASE ( View_MemberExpression )
+{
+    ParseAndVerify ( "version 2; table T#1 { column U8 c1 = 1; }; view W#1 <T t> { column U8 c2 = t . c1; }" );
+}
+
+TEST_CASE ( View_JoinExpression )
+{
+    ParseAndVerify ( "version 2; table T#1 { column U8 c1 = 1; }; view W#1 <T t> { column U8 c2 = t [ 1 ] . c1; }" );
 }
 
 //////////////////////////////////////////// Main
