@@ -22,15 +22,11 @@
 //
 // ===========================================================================
 
-#include <vdb/manager.h> // VDBManager
-#include <vdb/database.h>
-#include <vdb/table.h>
-#include <vdb/cursor.h>
+#include "VDB_Fixture.hpp"
+
 #include <vdb/blob.h>
 #include <vdb/vdb-priv.h>
 #include <vdb/blob.h>
-#include <sra/sraschema.h> // VDBManagerMakeSRASchema
-#include <vdb/schema.h> /* VSchemaRelease */
 
 extern "C" {
     #include <../libs/vdb/blob-priv.h>
@@ -43,7 +39,9 @@ extern "C" {
 #include <sysalloc.h>
 #include <cstdlib>
 #include <stdexcept>
-#include <map>
+#include <iostream>
+#include <sstream>
+#include <fstream>
 
 using namespace std;
 
@@ -194,163 +192,7 @@ TEST_CASE(SimultaneousCursors)
 }
 #endif
 
-class VdbFixture
-{
-public:
-    typedef std :: map< std :: string, const VDatabase * > Databases;
-
-public:
-    VdbFixture()
-    : mgr(0), curs(0)
-    {
-        if ( VDBManagerMakeRead(&mgr, NULL) != 0 )
-            throw logic_error ( "VdbFixture: VDBManagerMakeRead failed" );
-        if ( dbs == nullptr )
-        {
-            dbs = new Databases();
-        }
-    }
-
-    ~VdbFixture()
-    {
-        if ( mgr && VDBManagerRelease ( mgr ) != 0 )
-            throw logic_error ( "~VdbFixture: VDBManagerRelease failed" );
-        if ( curs && VCursorRelease ( curs ) != 0 )
-            throw logic_error ( "~VdbFixture: VCursorRelease failed" );
-    }
-
-    // call once per process
-    static void ReleaseCache()
-    {
-        for (auto d : *dbs )
-        {
-            VDatabaseRelease ( d.second );
-        }
-        delete dbs;
-        dbs = nullptr;
-    }
-
-    rc_t Setup( const char * acc, const char* column[], bool open = true )
-    {
-        rc_t rc = 0;
-        rc_t rc2;
-        const VDatabase *db = NULL;
-
-        std :: string ac(acc);
-        auto d = dbs->find(ac);
-        if ( d != dbs->end() )
-        {
-            db = d->second;
-            rc = VDatabaseAddRef( db );
-        }
-        else
-        {
-            rc = VDBManagerOpenDBRead ( mgr, &db, NULL, acc );
-            if ( rc == 0 )
-            {
-                dbs->insert( Databases::value_type( ac, db) );
-                rc = VDatabaseAddRef( db );
-            }
-        }
-
-        const VTable *tbl = NULL;
-        if (rc == 0)
-        {
-            rc = VDatabaseOpenTableRead ( db, &tbl, "SEQUENCE" );
-        }
-        else
-        {
-            rc = VDBManagerOpenTableRead ( mgr, &tbl, NULL, acc );
-            if (rc != 0)
-            {
-                VSchema *schema = NULL;
-                rc = VDBManagerMakeSRASchema(mgr, &schema);
-                if ( rc != 0 )
-                {
-                    return rc;
-                }
-
-                rc = VDBManagerOpenTableRead ( mgr, &tbl, schema, acc );
-
-                rc2 = VSchemaRelease ( schema );
-                if ( rc == 0 )
-                    rc = rc2;
-            }
-        }
-
-        if ( rc == 0 && column[0] != NULL )
-        {
-            rc = VTableCreateCursorRead(tbl, &curs);
-            if ( rc == 0 )
-            {
-                int i;
-                int const N = (int)(sizeof(col_idx)/sizeof(col_idx[0]));
-                for (i = 0; i < N; ++i)
-                    col_idx[i] = (uint32_t)(~0);
-                for (i = 0; column[i] != NULL && rc == 0; ++i) {
-                    assert(i < N);
-                    rc = VCursorAddColumn ( curs, col_idx + i, column[i] );
-                }
-                if ( rc == 0 && open )
-                {
-                    rc = VCursorOpen(curs);
-                }
-            }
-        }
-
-        rc2 = VTableRelease ( tbl );
-        if ( rc == 0 )
-            rc = rc2;
-
-        if ( db != NULL )
-        {
-            rc2 = VDatabaseRelease ( db );
-            if ( rc == 0 )
-               rc = rc2;
-        }
-
-        return rc;
-    }
-
-    bool CheckBlobRange ( int64_t p_rowId, int64_t p_first, uint64_t p_count )
-    {
-        THROW_ON_RC ( VCursorSetRowId ( curs, p_rowId ) );
-        THROW_ON_RC ( VCursorOpenRow ( curs ) );
-
-        struct VBlob const *blob;
-        THROW_ON_RC ( VCursorGetBlob ( curs, &blob, col_idx[0] ) );
-
-        bool ret = true;
-
-        int64_t first;
-        uint64_t count;
-        THROW_ON_RC ( VBlobIdRange ( blob, &first, &count ) );
-        if ( p_first != first )
-        {
-            cout << "CheckBlobRange(" << p_rowId << " ) : expected first = " << p_first << ", actual = " << first << endl;
-            ret = false;
-        }
-        if ( p_count != count )
-        {
-            cout << "CheckBlobRange(" << p_rowId << " ) : expected count = " << p_count << ", actual = " << count << endl;
-            ret = false;
-        }
-
-        THROW_ON_RC ( VCursorCloseRow (curs ) );
-        THROW_ON_RC ( VBlobRelease ( (struct VBlob *) blob ) );
-        return ret;
-    }
-
-
-    const VDBManager * mgr;
-    const VCursor * curs;
-    uint32_t col_idx[20];
-    static Databases * dbs;
-};
-
-VdbFixture::Databases * VdbFixture::dbs = nullptr;
-
-FIXTURE_TEST_CASE(TestCursorIsStatic_SingleRowRun1, VdbFixture)
+FIXTURE_TEST_CASE(TestCursorIsStatic_SingleRowRun1, VDB_Fixture)
 {
     static char const *columns[] = { "READ_LEN", 0 };
     REQUIRE_RC ( Setup ( "SRR002749", columns ) );
@@ -358,7 +200,7 @@ FIXTURE_TEST_CASE(TestCursorIsStatic_SingleRowRun1, VdbFixture)
     REQUIRE_RC ( VCursorIsStaticColumn ( curs, col_idx[0], &is_static) );
     REQUIRE ( is_static );
 }
-FIXTURE_TEST_CASE(TestCursorIsStatic_VariableREAD_LEN, VdbFixture)
+FIXTURE_TEST_CASE(TestCursorIsStatic_VariableREAD_LEN, VDB_Fixture)
 {
     static char const *columns[] = { "READ_LEN", 0 };
     REQUIRE_RC ( Setup ( "SRR050566", columns ) );
@@ -367,7 +209,7 @@ FIXTURE_TEST_CASE(TestCursorIsStatic_VariableREAD_LEN, VdbFixture)
     REQUIRE ( ! is_static );
 }
 #if 0
-FIXTURE_TEST_CASE(TestCursorIsStatic_SingleRowRun2, VdbFixture)
+FIXTURE_TEST_CASE(TestCursorIsStatic_SingleRowRun2, VDB_Fixture)
 {
     static char const *columns[] = { "READ_LEN", 0 };
     REQUIRE_RC ( Setup ( "SRR053325", columns ) );
@@ -376,7 +218,7 @@ FIXTURE_TEST_CASE(TestCursorIsStatic_SingleRowRun2, VdbFixture)
     REQUIRE ( is_static );
 }
 #endif
-FIXTURE_TEST_CASE(TestCursorIsStatic_FixedREAD_LEN_MultipleRows, VdbFixture)
+FIXTURE_TEST_CASE(TestCursorIsStatic_FixedREAD_LEN_MultipleRows, VDB_Fixture)
 {
     static char const *columns[] = { "READ_LEN", 0 };
     REQUIRE_RC ( Setup ( "SRR125365", columns ) );
@@ -384,7 +226,7 @@ FIXTURE_TEST_CASE(TestCursorIsStatic_FixedREAD_LEN_MultipleRows, VdbFixture)
     REQUIRE_RC ( VCursorIsStaticColumn ( curs, col_idx[0], &is_static) );
     REQUIRE ( is_static );
 }
-FIXTURE_TEST_CASE(TestCursorIsStatic_DB_FixedREAD_LEN_MultipleRows, VdbFixture)
+FIXTURE_TEST_CASE(TestCursorIsStatic_DB_FixedREAD_LEN_MultipleRows, VDB_Fixture)
 {
     static char const *columns[] = { "READ_LEN", 0 };
     REQUIRE_RC ( Setup ( "SRR600096", columns ) );
@@ -392,7 +234,7 @@ FIXTURE_TEST_CASE(TestCursorIsStatic_DB_FixedREAD_LEN_MultipleRows, VdbFixture)
     REQUIRE_RC ( VCursorIsStaticColumn ( curs, col_idx[0], &is_static) );
     REQUIRE ( is_static );
 }
-FIXTURE_TEST_CASE(TestCursorIsStatic_DB_VariableREAD_LEN_MultipleRows, VdbFixture)
+FIXTURE_TEST_CASE(TestCursorIsStatic_DB_VariableREAD_LEN_MultipleRows, VDB_Fixture)
 {
     static char const *columns[] = { "READ_LEN", 0 };
     REQUIRE_RC ( Setup ( "SRR619505", columns ) );
@@ -403,7 +245,7 @@ FIXTURE_TEST_CASE(TestCursorIsStatic_DB_VariableREAD_LEN_MultipleRows, VdbFixtur
 
 /////////////////// VBlob
 
-FIXTURE_TEST_CASE(VCursor_GetBlob_SRA, VdbFixture)
+FIXTURE_TEST_CASE(VCursor_GetBlob_SRA, VDB_Fixture)
 {   // multiple fragments per row (some are technical), multiple rows per blob
     static char const *columns[] = { "READ", 0 };
     REQUIRE_RC ( Setup ( "SRR000123", columns ) );
@@ -473,7 +315,7 @@ FIXTURE_TEST_CASE(VCursor_GetBlob_SRA, VdbFixture)
     }
 }
 
-FIXTURE_TEST_CASE(VCursor_GetBlob_WGS, VdbFixture)
+FIXTURE_TEST_CASE(VCursor_GetBlob_WGS, VDB_Fixture)
 {   // single fragment per row, multiple rows per blob
     static char const *columns[] = { "READ", 0 };
     REQUIRE_RC ( Setup ( "ALWZ01", columns ) );
@@ -543,7 +385,7 @@ FIXTURE_TEST_CASE(VCursor_GetBlob_WGS, VdbFixture)
     }
 }
 
-FIXTURE_TEST_CASE(VCursor_GetBlob_SequentialAccess, VdbFixture)
+FIXTURE_TEST_CASE(VCursor_GetBlob_SequentialAccess, VDB_Fixture)
 {   // VDB-2858: sequential access to blobs broken
     static char const *columns[] = { "READ", 0 };
     REQUIRE_RC ( Setup ( "ALAI01", columns ) );
@@ -575,7 +417,40 @@ FIXTURE_TEST_CASE(VCursor_GetBlob_SequentialAccess, VdbFixture)
     }
 }
 
-FIXTURE_TEST_CASE(VCursor_GetBlob_RandomAccess, VdbFixture)
+class VdbBlobFixture : public VDB_Fixture
+{
+public:
+    bool CheckBlobRange ( int64_t p_rowId, int64_t p_first, uint64_t p_count )
+    {
+        THROW_ON_RC ( VCursorSetRowId ( curs, p_rowId ) );
+        THROW_ON_RC ( VCursorOpenRow ( curs ) );
+
+        struct VBlob const *blob;
+        THROW_ON_RC ( VCursorGetBlob ( curs, &blob, col_idx[0] ) );
+
+        bool ret = true;
+
+        int64_t first;
+        uint64_t count;
+        THROW_ON_RC ( VBlobIdRange ( blob, &first, &count ) );
+        if ( p_first != first )
+        {
+            cout << "CheckBlobRange(" << p_rowId << " ) : expected first = " << p_first << ", actual = " << first << endl;
+            ret = false;
+        }
+        if ( p_count != count )
+        {
+            cout << "CheckBlobRange(" << p_rowId << " ) : expected count = " << p_count << ", actual = " << count << endl;
+            ret = false;
+        }
+
+        THROW_ON_RC ( VCursorCloseRow (curs ) );
+        THROW_ON_RC ( VBlobRelease ( (struct VBlob *) blob ) );
+        return ret;
+    }
+};
+
+FIXTURE_TEST_CASE(VCursor_GetBlob_RandomAccess, VdbBlobFixture)
 {
     static char const *columns[] = { "READ", 0 };
     REQUIRE_RC ( Setup ( "SRR000001", columns ) );
@@ -592,7 +467,7 @@ FIXTURE_TEST_CASE(VCursor_GetBlob_RandomAccess, VdbFixture)
     REQUIRE ( CheckBlobRange ( 14001, 14001, 1 ) );
 }
 
-FIXTURE_TEST_CASE(PageMapIterator_WGS, VdbFixture)
+FIXTURE_TEST_CASE(PageMapIterator_WGS, VDB_Fixture)
 {   // single fragment per row, multiple rows per blob
     static char const *columns[] = { "READ", 0 };
     REQUIRE_RC ( Setup ( "ALWZ01", columns ) );
@@ -632,7 +507,7 @@ FIXTURE_TEST_CASE(PageMapIterator_WGS, VdbFixture)
     }
 }
 
-FIXTURE_TEST_CASE ( VCursor_FindNextRowIdDirect, VdbFixture )
+FIXTURE_TEST_CASE ( VCursor_FindNextRowIdDirect, VDB_Fixture )
 {
     static char const *columns[] = { "SPOT_ID", "READ", 0 };
     REQUIRE_RC ( Setup ( "SRR000001", columns ) );
@@ -643,7 +518,7 @@ FIXTURE_TEST_CASE ( VCursor_FindNextRowIdDirect, VdbFixture )
     REQUIRE_EQ ( (int64_t)2, next ) ; // VDB-3075: next == 1
 }
 
-FIXTURE_TEST_CASE ( DoubleOpen, VdbFixture )
+FIXTURE_TEST_CASE ( DoubleOpen, VDB_Fixture )
 {
     const char SRR619505[] = "SRR619505";
 
@@ -664,13 +539,47 @@ FIXTURE_TEST_CASE ( DoubleOpen, VdbFixture )
     REQUIRE_RC ( VDatabaseRelease ( db ) );
 }
 
-FIXTURE_TEST_CASE(VCursor_PermitPostOpenAdd, VdbFixture)
+FIXTURE_TEST_CASE(VCursor_PermitPostOpenAdd, VDB_Fixture)
 {
     static char const *columns[] = { "SPOT_ID", 0 };
     REQUIRE_RC(Setup("SRR000001", columns, false));
     REQUIRE_RC(VCursorPermitPostOpenAdd(curs));
 }
 
+#if TO_FIX_VDB_4470
+static rc_t CC FlushSchema ( void *fd, const void * buffer, size_t size )
+{
+    string * strPtr = static_cast<string *>( fd );
+    strPtr -> append ( static_cast<const char*>( buffer ), size );
+    return 0;
+}
+
+FIXTURE_TEST_CASE(VersionProblem, VDB_Fixture)
+{
+    const string input =
+    "version 1; table NCBI:tbl:base_space #3 {}; "
+    " table NCBI:align:tbl:reference #3 = NCBI:tbl:base_space #3 {}; "
+    " table NCBI:tbl:base_space #3.1 {};";
+
+    string out;
+    {
+        VSchema * schema;
+        REQUIRE_RC ( VDBManagerMakeSchema ( mgr, & schema ) );
+        REQUIRE_RC ( VDBManagerAddSchemaIncludePath ( mgr, "%s", "../../interfaces" ) );
+        REQUIRE_RC ( VSchemaParseText ( schema, NULL, input . c_str(), input . size () ) );
+        REQUIRE_RC ( VSchemaDump ( schema, sdmPrint, "NCBI:align:tbl:reference #3", FlushSchema, & out ) );
+        VSchemaRelease ( schema );
+    }
+cout << out;
+    {
+        VSchema * schema;
+        REQUIRE_RC ( VDBManagerMakeSchema ( mgr, & schema ) );
+        REQUIRE_RC ( VSchemaParseText ( schema, NULL, out . c_str(), out . size () ) );
+        VSchemaRelease ( schema );
+    }
+}
+#endif
+
 //////////////////////////////////////////// Main
 extern "C"
 {
