@@ -24,6 +24,7 @@
 *
 */
 
+#include <kfg/config.h> /* KConfigDisableUserSettings */
 #include <klib/debug.h> /* KDbgSetString */
 #include <ktst/unit_test.hpp> // TEST_SUITE
 #include <vfs/path.h> /* VPathRelease */
@@ -75,8 +76,7 @@ TEST_CASE(testArrayExample) {
             "]"
         "}"
     "]"
-"}"
-,false) );
+"}"));
     REQUIRE_RC      ( Response4Release ( response ) );
 }
 #endif
@@ -119,8 +119,7 @@ TEST_CASE(testExample) {
             "]"
         "}"
     "]"
-"}"
-, false) );
+"}") );
     REQUIRE_RC      ( Response4Release ( response ) );
 }
 #endif
@@ -128,7 +127,7 @@ TEST_CASE(testExample) {
 #ifdef ALL
 TEST_CASE(doubleWhack) {
     Response4 * response = NULL;
-    REQUIRE_RC_FAIL(Response4MakeSdl(&response, "{}", false));
+    REQUIRE_RC_FAIL(Response4MakeSdl(&response, "{}"));
     REQUIRE_RC(Response4Release(response));
 }
 #endif
@@ -312,7 +311,7 @@ TEST_CASE(nullMsg) {
     REQUIRE_RC_FAIL(Response4MakeSdl(&response,
         "{"
         "    \"status\": 200"
-        "}", false));
+        "}"));
 
     REQUIRE_RC(Response4Release(response));
 }
@@ -327,7 +326,7 @@ TEST_CASE(noMsg) {
         "{"
         "    \"status\": 200,"
         "    \"message\" : \"\""
-        "}", false));
+        "}"));
 
     rc_t rc = 0;
     REQUIRE_RC(Response4GetRc(response, &rc));
@@ -348,7 +347,7 @@ TEST_CASE(noMsgWithRes) {
         "    {"
         "    }"
         "  ]"
-        "}", false));
+        "}"));
 
     REQUIRE_RC(Response4Release(response));
 }
@@ -363,7 +362,7 @@ TEST_CASE(invalidExpiration) {
         "{"
         "    \"status\": 200,"
         "    \"message\" : \"Claims have expired\""
-        "}", false));
+        "}"));
 
     rc_t rc = 0;
     REQUIRE_RC(Response4GetRc(response, &rc));
@@ -382,7 +381,7 @@ TEST_CASE(invalidMsg) {
         "{"
         "    \"status\": 200,"
         "    \"msg\" : \"Signature not recognized\""
-        "}", false));
+        "}"));
 
     rc_t rc = 0;
     REQUIRE_RC(Response4GetRc(response, &rc));
@@ -401,7 +400,7 @@ TEST_CASE(expired) {
         "{"
         "    \"status\": 440,"
         "    \"message\" : \"Claims have expired\""
-        "}", false));
+        "}"));
     
     rc_t rc = 0;
     REQUIRE_RC(Response4GetRc(response, &rc));
@@ -420,7 +419,7 @@ TEST_CASE(expiredNsg) {
         "{"
         "    \"status\": 440,"
         "    \"msg\" : \"Claims have expired\""
-        "}", false));
+        "}"));
 
     rc_t rc = 0;
     REQUIRE_RC(Response4GetRc(response, &rc));
@@ -439,6 +438,7 @@ if (
 0 ) assert ( ! KDbgSetString ( "VFS" ) );
 if (
 0 ) assert ( ! KDbgSetString ( "VFS-JSON" ) );
+    KConfigDisableUserSettings();
     return TestSdlResolver( argc, argv );
     }
 }
